#pragma once
#include <cstdint>
#include <memory>
#include <collections.hpp>
#include "../lua/FutureLua.hpp"
#include <TickSystem.hpp>
#include "../LuaSortList.hpp"

class Bubble_Sorter : Future<LuaSortList>
{
	LuaSortList* list = nullptr;
	struct STATE
	{
		size_t n = 1;
	} state;
public:
	Bubble_Sorter() = default;
	Bubble_Sorter(LuaSortList& l) : list(&l) {}

	bool poll() override
	{
		size_t n = list->size();
		if (n <= 1) {
			return true;
		}

		state.n = state.n + 1;

		if (state.n > n)
		{
			state.n = 2;
		}

		if (list->at(state.n - 2) > list->at(state.n - 1)) {
			list->swap(state.n - 2, state.n - 1);
			return false;
		}
		return list->sorted();
	}

	LuaSortList& get() override
	{
		return *list;
	}

	void set(LuaSortList& l)
	{
		list = &l;
	}
};

class Sortiva final
{
public:
	Sortiva();
	~Sortiva();

	void run();
private:
	int m_Width;
	int m_Height;
	bool m_Running = true;

	void draw();
	void draw_overlay();
	void update(double);

	void setup();

	bool m_SortingFinished = true;
	TickSystem m_Ticker;
	Bubble_Sorter m_Sorter;
	LuaSortList m_List;


	//	std::unique_ptr
	//		-> tree 
	//			-> linked_list
	//				-> uint16_t
	using val_step_diag = std::vector<linked_list<uint16_t>>;
	std::unique_ptr<val_step_diag> m_Steps;
};
